# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

import typing
from pathlib import Path

import frontmatter
import msgspec

from .models import Document, Section


def load_document(file: typing.IO):
    return msgspec.yaml.decode(file.read(), type=Document)


def build_page(text: str) -> str:
    metadata, content = frontmatter.parse(text)
    return "# {}\n\n{}".format(metadata["title"], content).rstrip()


def build_section(doc_dir: Path, section: Section) -> str:
    return build_page(open(doc_dir / section.file).read())


def build_document(doc_dir: Path, document: Document) -> str:
    output = "\n\n".join(
        [build_section(doc_dir, section) for section in document.sections]
    )
    text = frontmatter.Post(output, title=document.title, version=document.version)
    return frontmatter.dumps(text) + "\n"



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

import msgspec


class Section(msgspec.Struct, frozen=True):
    file: str


class Document(msgspec.Struct, frozen=True):
    title: str
    version: str

    metadata: dict[str, str] = msgspec.field(default_factory=dict)

    sections: list[Section] = msgspec.field(default_factory=list)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

import argparse
import sys
from pathlib import Path

from .builder import build_document, load_document


def main():
    parser = argparse.ArgumentParser(prog="squishie")
    parser.add_argument(
        "-c",
        "--config",
        required=True,
        type=argparse.FileType("r"),
        help="document config file path",
    )
    parser.add_argument(
        "-o",
        "--output",
        default=sys.stdout,
        type=argparse.FileType("w"),
        help="output file path",
    )
    args = parser.parse_args()

    document = load_document(args.config)
    output = build_document(Path(args.config.name).parent, document)
    args.output.write(output)



# SPDX-FileCopyrightText: UL Research Institutes
# SPDX-License-Identifier: Apache-2.0

from pathlib import Path

from squishie.builder import build_document, load_document

FIXTURE_DIR = Path(__file__).parent / "fixtures"


def test_load_document():
    document_file = FIXTURE_DIR / "minimal" / "squishie.yaml"
    document = load_document(open(document_file))
    assert document.title == "My squished document"
    assert document.version == "1.2.3"
    assert document.sections[0].file == "doc1.md"
    assert document.sections[1].file == "doc2.md"


def test_build_document():
    document_file = FIXTURE_DIR / "minimal" / "squishie.yaml"
    document = load_document(open(document_file))
    output = build_document(FIXTURE_DIR / "minimal", document)
    expected = """---
title: My squished document
version: 1.2.3
---

# Doc 1

This is some text.

# Doc 2

This is different text.
"""
    assert output == expected
